<?php

namespace Drupal\watchdog_mailer\Logger;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Database\Connection;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Utility\Token;
use Psr\Log\LoggerInterface;

/**
 * {@inheritdoc}
 */
class WatchdogMailer implements LoggerInterface {

  use RfcLoggerTrait;
  use StringTranslationTrait;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * The (lazy loaded) dependency injection (DI) container.
   *
   * @var ?\Drupal\Component\DependencyInjection\ContainerInterface
   */
  protected ?ContainerInterface $container;

  /**
   * The (lazy loaded) mail manager object.
   *
   * @var ?\Drupal\Core\Mail\MailManagerInterface
   */
  protected ?MailManagerInterface $mailManager;

  /**
   * The (lazy loaded) token object.
   *
   * @var ?\Drupal\Core\Utility\Token
   */
  protected ?Token $token;

  /**
   * Constructs a new WatchdogMailer.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   A config factory for retrieving required config objects.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection to run the query against.
   * @param \Drupal\Component\Datetime\TimeInterface $timeInterface
   *   The time interface service.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              protected LanguageManagerInterface $languageManager,
                              protected Connection $connection,
                              protected TimeInterface $timeInterface) {
    $this->config = $config_factory->get('watchdog_mailer.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function log($level, $message, array $context = []): void {
    $enabled = $this->config->get('watchdog_mailer_enable') ?? FALSE;
    $channelsEnable = $this->config->get('channels_enable') ?? FALSE;
    $channelsNegate = $this->config->get('channels_negate') ?? FALSE;
    $channelsList = $this->config->get('channels_list') ?? '';
    $maxEmails = $this->config->get('max_emails_per_hour') ?? 0;
    $channels = explode("\r\n", mb_strtolower($channelsList));

    if ($channelsEnable && !empty($channels)) {
      if (!$channelsNegate && !in_array($context['channel'] ?: '', $channels) || $channelsNegate && in_array($context['channel'] ?: '', $channels)) {
        $enabled = FALSE;
      }
    }
    if (!empty($maxEmails)) {
      $query = $this->connection->select('watchdog_mailer_history', 'w')
        ->condition('w.timestamp', $this->timeInterface->getRequestTime() - 3600, '>')
        ->countQuery();
      $count = $query->execute()->fetchField();
      if ($count >= $maxEmails) {
        $enabled = FALSE;
      }
    }
    if ($enabled) {
      try {
        if ($level <= RfcLogLevel::WARNING || ($this->config->get('php_notices') && $context['channel'] === 'php')) {
          $body = $this->config->get('mail_body');
          $message = new FormattableMarkup($message, $context);
          $tokenValues = [
            'watchdog_mailer' => [
              // @todo Check if the escaping is really needed:
              'channel' => Html::escape($context['channel']),
              'timestamp' => Html::escape($context['timestamp']),
              'referrer' => isset($context['referer']) ? Html::escape($context['referer']) : '',
              'link' => isset($context['link']) ? Html::escape($context['link']) : '',
              'uid' => Html::escape($context['uid']),
              'user_ip' => Html::escape($context['ip']),
              'message' => $message,
              // @todo Would be nice to get the name of the log level here,
              // but sadly class Logger doesn't seem to provide
              // a public mapping or function for that?
              // At least for PHP we could use one of:
              // - https://api.drupal.org/api/drupal/core%21includes%21errors.inc/function/drupal_error_levels/10
              // - https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Core%21Logger%21RfcLogLevel.php/function/RfcLogLevel%3A%3AgetLevels/10
              // These are only available for PHP errors:
              'php_severity_level' => isset($context['severity_level']) ? Html::escape($context['severity_level']) : '',
              'php_backtrace' => isset($context['@backtrace_string']) ? Html::escape($context['@backtrace_string']) : '',
              'php_file' => isset($context['%file']) ? Html::escape($context['%file']) : '',
              'php_function' => isset($context['%function']) ? Html::escape($context['%function']) : '',
              'php_line' => isset($context['%line']) ? Html::escape($context['%line']) : '',
              'php_message' => isset($context['@message']) ? Html::escape($context['@message']) : '',
              'php_type' => isset($context['%type']) ? Html::escape($context['%type']) : '',
            ],
          ];
          $body = $this->token()->replace($body, $tokenValues, ['clear' => TRUE]);
          $body = Html::decodeEntities(strip_tags($body));
          $subject = $this->config->get('mail_subject');
          $subject = $this->token()->replace($subject, $tokenValues, ['clear' => TRUE]);
          $params = [
            'subject' => $subject,
            'body' => $body,
          ];
          $langcode = $this->languageManager->getCurrentLanguage()->getId();
          $mailAddresses = explode(',', $this->config->get('mail_addresses'));
          foreach ($mailAddresses as $mailAddress) {
            $mailAddress = trim($mailAddress);
            if (!empty($mailAddress)) {
              if (!$this->mailManager()->mail(
                'watchdog_mailer',
                'watchdog_mailer',
                $mailAddress,
                $langcode,
                $params)
              ) {
                throw new \Exception('There was a problem sending the message.');
              }
              else {
                $this->connection->insert('watchdog_mailer_history')
                  ->fields(['timestamp' => time()])
                  ->execute();
              }
            }
          }
        }
      }
      catch (\Exception $e) {
        // There can be no watchdog_exception to avoid a possible endless cycle.
        $this->error($e->getMessage());
      }
    }
  }

  /**
   * Get Dependency Injection container.
   *
   * @return \Drupal\Component\DependencyInjection\ContainerInterface
   *   Current Dependency Injection container.
   */
  protected function getContainer(): ContainerInterface {
    if (!isset($this->container)) {
      $this->container = \Drupal::getContainer();
    }
    return $this->container;
  }

  /**
   * Get lazy-loaded Mailmanager service.
   *
   * @see https://www.drupal.org/project/watchdog_mailer/issues/3337719
   *
   * @return \Drupal\Core\Mail\MailManagerInterface
   *   MailManager service.
   */
  protected function mailManager(): MailManagerInterface {
    if (!isset($this->mailManager)) {
      $this->mailManager = $this->getContainer()->get('plugin.manager.mail');
    }
    return $this->mailManager;
  }

  /**
   * Get lazy-loaded Token service.
   *
   * @see https://www.drupal.org/project/watchdog_mailer/issues/3414001
   *
   * @return \Drupal\Core\Utility\Token
   *   Token service.
   */
  protected function token(): Token {
    if (!isset($this->token)) {
      $this->token = $this->getContainer()->get('token');
    }
    return $this->token;
  }
}
