<?php

namespace Drupal\Tests\watchdog_mailer\Functional;

/**
 * This class provides methods specifically for testing something.
 *
 * @group watchdog_mailer
 */
class WatchdogMailerMailTest extends WatchdogMailerTestBase {

  /**
   * Tests if a mail is sent to the email address specified in the settings.
   */
  public function testMailSingle() {
    $this->drupalGet('/admin/config/development/logging');
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'foo@bar.baz',
      'mail_subject' => 'mail test subject',
      'mail_body' => 'Mail test body.',
      'max_emails_per_hour' => 100,
      'php_notices' => TRUE,
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->rebuildContainer();
    // Produce a log entry:
    \Drupal::service('logger.factory')
      ->get('php')
      ->error('Error in the database log: php');
    $mails = $this->getMails(['module' => 'watchdog_mailer']);
    $this->assertCount(1, $mails);
    $this->assertMailString('subject', $edit['mail_subject'], 1);
    $this->assertMailString('body', $edit['mail_body'], 1);
  }

  /**
   * Test email sending with multiple destination email-address.
   */
  public function testMailMultiple() {
    $this->drupalGet('/admin/config/development/logging');
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'foo@bar.baz, example@example.com',
      'max_emails_per_hour' => 100,
      'php_notices' => TRUE,
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->rebuildContainer();
    // Produce a log entry:
    \Drupal::service('logger.factory')
      ->get('php')
      ->error('Error in the database log: php');
    $mails = $this->getMails(['module' => 'watchdog_mailer']);
    $this->assertCount(2, $mails);
    $this->assertMailString('to', 'foo@bar.baz', 2);
    $this->assertMailString('to', 'example@example.com', 2);
  }

  /**
   * Test channel filtering.
   */
  public function testChannelFiltering() {
    $this->drupalGet('/admin/config/development/logging');
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'example@example.com',
      'channels_enable' => TRUE,
      'channels_list' => "foo\r\nbaz",
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->rebuildContainer();
    // Produce a notice log entry:
    foreach (['foo', 'bar', 'baz', 'php'] as $channel) {
      \Drupal::service('logger.factory')
        ->get($channel)
        ->notice("Error in the database log: $channel");
    }
    // Test mails with notice log entry.
    $mails = $this->getMails(['module' => 'watchdog_mailer']);
    $this->assertCount(0, $mails);
    // Produce a error log entry:
    foreach (['foo', 'bar', 'baz', 'php'] as $channel) {
      \Drupal::service('logger.factory')
        ->get($channel)
        ->error("Error in the database log: $channel");
    }
    // Test mails with error log entry.
    $mails = $this->getMails(['module' => 'watchdog_mailer']);
    $this->assertCount(2, $mails);
    $this->assertMailString('subject', 'Error in the database log: foo', 2);
    $this->assertMailString('subject', 'Error in the database log: baz', 2);
  }

  /**
   * Test channel filtering negation.
   */
  public function testChannelFilteringNegate() {
    $this->drupalGet('/admin/config/development/logging');
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'example@example.com',
      'channels_enable' => TRUE,
      'channels_list' => "foo\r\nbaz",
      'channels_negate' => TRUE,
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->rebuildContainer();
    // Produce a notice log entry:
    foreach (['foo', 'bar', 'baz', 'php'] as $channel) {
      \Drupal::service('logger.factory')
        ->get($channel)
        ->notice("Error in the database log: $channel");
    }
    // Test mails with notice log entry.
    $mails = $this->getMails(['module' => 'watchdog_mailer']);
    $this->assertCount(0, $mails);
    // Produce a error log entry:
    foreach (['foo', 'bar', 'baz', 'php'] as $channel) {
      \Drupal::service('logger.factory')
        ->get($channel)
        ->error("Error in the database log: $channel");
    }
    // Test mails with error log entry.
    $mails = $this->getMails(['module' => 'watchdog_mailer']);
    $this->assertCount(2, $mails);
    $this->assertMailString('subject', 'Error in the database log: bar', 2);
    $this->assertMailString('subject', 'Error in the database log: php', 2);
  }

  /**
   * Test logging PHP notices.
   */
  public function testPHPNotices() {
    $this->drupalGet('/admin/config/development/logging');
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'example@example.com',
      'php_notices' => TRUE,
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->rebuildContainer();
    foreach (['emergency', 'alert', 'critical', 'error', 'warning', 'notice', 'info', 'debug'] as $value) {
      \Drupal::service('logger.factory')
        ->get('php')
        ->$value("Error in the database log: PHP $value");
    }
    // Test mails.
    $mails = $this->getMails(['module' => 'watchdog_mailer']);
    $this->assertCount(8, $mails);
  }
}
